<?php
/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://makewebbetter.com/
 * @since      1.0.0
 *
 * @package    Mwb_Role_Based_Pricing_For_Woocommerce
 * @subpackage Mwb_Role_Based_Pricing_For_Woocommerce/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 * namespace mwb_role_based_pricing_for_woocommerce_public.
 *
 * @package    Mwb_Role_Based_Pricing_For_Woocommerce
 * @subpackage Mwb_Role_Based_Pricing_For_Woocommerce/public
 * @author     makewebbetter <webmaster@makewebbetter.com>
 */
class Mwb_Role_Based_Pricing_For_Woocommerce_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string $plugin_name       The name of the plugin.
	 * @param      string $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function mwb_mrbpfw_public_enqueue_styles() {

		wp_enqueue_style( $this->plugin_name, MWB_ROLE_BASED_PRICING_FOR_WOOCOMMERCE_DIR_URL . 'public/css/mwb-role-based-pricing-for-woocommerce-public.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function mwb_mrbpfw_public_enqueue_scripts() {

		wp_register_script( $this->plugin_name, MWB_ROLE_BASED_PRICING_FOR_WOOCOMMERCE_DIR_URL . 'public/js/mwb-role-based-pricing-for-woocommerce-public.js', array( 'jquery' ), $this->version, false );
		wp_localize_script( $this->plugin_name, 'mrbpfw_public_param', array( 'ajaxurl' => admin_url( 'admin-ajax.php' ) ) );
		wp_enqueue_script( $this->plugin_name );

	}

	/**
	 * Function will return the price
	 *
	 * @param string $original_price is current product product price.
	 * @param object $product is the current product object.
	 * @return $price or $original_price which can be modify price or original price based the conditions
	 */
	public function mwb_mrbpfw_my_custom_price( $original_price, $product ) {
		global $post, $woocommerce;
		if ( is_admin() ) {
			return $original_price;
		}
		if ( class_exists( 'WC_Subscriptions_Product' ) && $product->is_subscription() ) {
			return $original_price;
		}
		$price = $this->mwb_mrbpfw_role_based_price( $original_price, $product, 'simple' );
		if ( ! empty( $price ) ) {
			return $price;
		} else {
			return $original_price;
		}
	}

	/**
	 * Function to get the all rule ids depending upon the current user role.
	 */
	public function mwb_mrbpfw_get_all_rule_ids_for_current_role() {
		// Logic for calculating the new price here.
		if ( ! is_user_logged_in() ) {
			$current_role = array( 'guest' );
		} else {
			$get_current_user_role = wp_get_current_user();
			$current_role          = $get_current_user_role->roles;
		}
		$all_rules_ids = get_posts(
			array(
				'fields'         => 'ids',
				'posts_per_page' => -1,
				'post_type'      => 'mrbpfw_price_rules',
				'meta_query'     => array(
					array(
						'key'     => 'mwb_mrbpfw_role',
						'value'   => $current_role[0],
						'compare' => '==',
					),
					array(
						'key'     => 'mwb_mrbpfw_enable_rule',
						'value'   => 'on',
						'compare' => '==',
					),
				),
			)
		);
		return $all_rules_ids;
	}

	/**
	 * Function to check if discount avaiable for the current product
	 *
	 * @param object $product is a product object.
	 * @return $discounts
	 */
	public function mwb_mrbpfw_discount_availability_for_product( $product ) {
		if ( $product->is_type( 'simple' ) ) {
			$product_id = $product->get_id();
		} elseif ( $product->is_type( 'variable' ) || $product->is_type( 'variation' ) ) {
			$product_id = $product->get_parent_id();
		}
		if ( 'simple' === $product->get_type() || 'variable' === $product->get_type() || 'variation' === $product->get_type() ) {
			$all_rules_ids = $this->mwb_mrbpfw_get_all_rule_ids_for_current_role();
			if ( isset( $all_rules_ids ) && ! empty( $all_rules_ids ) ) {
				foreach ( $all_rules_ids as $key => $id ) {
					$priority                           = get_post_meta( $id, 'mwb_mrbpfw_priority', true );
					$priority_wise_rule_id[ $priority ] = $id;
				}
				ksort( $priority_wise_rule_id );
				$count = 0;
				foreach ( $priority_wise_rule_id as $key => $id ) {
					++$count;
					$rule_type       = get_post_meta( $id, 'mwb_mrbpfw_rule_type', true );
					$discount_type   = get_post_meta( $id, 'mwb_mrbpfw_discount_type', true );
					$price           = get_post_meta( $id, 'mwb_mrbpfw_price', true );
					$all_product_ids = get_post_meta( $id, 'mwb_mrbpfw_all_products', true );
					if ( isset( $rule_type ) && ! empty( $rule_type ) && 'all_products' === $rule_type && ( empty( $all_product_ids ) || ( ! empty( $all_product_ids ) && in_array( $product_id, $all_product_ids, false ) ) ) ) {
						if ( isset( $price ) && ! empty( $price ) ) {
							if ( 'fixed' === $discount_type ) {
								$discount = 'yes';
							} elseif ( 'percentage' === $discount_type ) {
								$discount = 'yes';
							}
						}
					} elseif ( isset( $rule_type ) && ! empty( $rule_type ) && 'categories' === $rule_type ) {
						$categories = get_post_meta( $id, 'mwb_mrbpfw_categories', true );
						if ( isset( $categories ) && ! empty( $categories ) ) {
							if ( has_term( $categories, 'product_cat', $product->get_id() ) ) {
								if ( 'fixed' === $discount_type ) {
									$discount = 'yes';
								} elseif ( 'percentage' === $discount_type ) {
									$discount = 'yes';
								}
							}
						}
					} elseif ( isset( $rule_type ) && ! empty( $rule_type ) && 'tags' === $rule_type ) {
						$tags = get_post_meta( $id, 'mwb_mrbpfw_tags', true );
						if ( isset( $tags ) && ! empty( $tags ) ) {
							if ( has_term( $tags, 'product_tag', $product->get_id() ) ) {
								if ( isset( $price ) && ! empty( $price ) ) {
									if ( 'fixed' === $discount_type ) {
										$discount = 'yes';
									} elseif ( 'percentage' === $discount_type ) {
										$discount = 'yes';
									}
								}
							}
						}
					}
				}
			}
		}
		if ( isset( $discount ) && ! empty( $discount ) ) {
			return $discount;
		} else {
			return false;
		}
	}

	/**
	 * Get Total discount for the products
	 *
	 * @param object $product .
	 * @param string $new_price .
	 */
	public function mwb_mrbpfw_get_discount_price( $product, $new_price ) {
		$all_rules_ids         = $this->mwb_mrbpfw_get_all_rule_ids_for_current_role();
		$priority_wise_rule_id = array();
		$total_discount        = '';
		if ( isset( $all_rules_ids ) && ! empty( $all_rules_ids ) && ! empty( $new_price ) ) {
			foreach ( $all_rules_ids as $key => $id ) {
				$priority                           = get_post_meta( $id, 'mwb_mrbpfw_priority', true );
				$priority_wise_rule_id[ $priority ] = $id;
			}
			ksort( $priority_wise_rule_id );
			$count = 0;
			foreach ( $priority_wise_rule_id as $key => $id ) {
				++$count;
				$rule_type       = get_post_meta( $id, 'mwb_mrbpfw_rule_type', true );
				$discount_type   = get_post_meta( $id, 'mwb_mrbpfw_discount_type', true );
				$price           = get_post_meta( $id, 'mwb_mrbpfw_price', true );
				$all_product_ids = get_post_meta( $id, 'mwb_mrbpfw_all_products', true );
				floatval( $price );
				if ( $product->is_type( 'variable' ) ) {
					$product_id = $product->get_parent_id();
				} elseif ( $product->is_type( 'variation' ) ) {
					$product_id = $product->get_parent_id();
				} elseif ( $product->is_type( 'simple' ) ) {
					$product_id = $product->get_id();
				}
				if ( isset( $rule_type ) && ! empty( $rule_type ) && 'all_products' === $rule_type && ( empty( $all_product_ids ) || ( ! empty( $all_product_ids ) && in_array( $product_id, $all_product_ids, false ) ) ) ) {
					if ( isset( $price ) && ! empty( $price ) ) {
						if ( 'fixed' === $discount_type ) {
							$total_discount = $price;
						} elseif ( 'percentage' === $discount_type ) {
							$per_price = ( $new_price * $price ) / 100;
							round( $per_price, 2 );
							$total_discount = $per_price;
						}
					}
				} elseif ( isset( $rule_type ) && ! empty( $rule_type ) && 'categories' === $rule_type ) {
					$categories = get_post_meta( $id, 'mwb_mrbpfw_categories', true );
					if ( isset( $categories ) && ! empty( $categories ) ) {
						if ( has_term( $categories, 'product_cat', $product->get_parent_id() ) ) {
							if ( isset( $price ) && ! empty( $price ) ) {
								if ( 'fixed' === $discount_type ) {
									$total_discount = $price;
								} elseif ( 'percentage' === $discount_type ) {
									$per_price = ( $new_price * $price ) / 100;
									round( $per_price, 2 );
									$total_discount = $per_price;
								}
							}
						}
					}
				} elseif ( isset( $rule_type ) && ! empty( $rule_type ) && 'tags' === $rule_type ) {
					$tags = get_post_meta( $id, 'mwb_mrbpfw_tags', true );
					if ( isset( $tags ) && ! empty( $tags ) ) {
						if ( has_term( $tags, 'product_tag', $product->get_parent_id() ) ) {
							if ( isset( $price ) && ! empty( $price ) ) {
								if ( 'fixed' === $discount_type ) {
									$total_discount = $price;
								} elseif ( 'percentage' === $discount_type ) {
									$per_price = ( $new_price * $price ) / 100;
									round( $per_price, 2 );
									$total_discount = $per_price;
								}
							}
						}
					}
				}
				if ( 1 === $count ) {
					break;
				}
			}
		}
		return $total_discount;
	}

	/**
	 * Function to change the product price for the product
	 *
	 * @param string $original_price is current product product price.
	 * @param object $product is the current product object.
	 * @param object $type .
	 * @return $new_price which can be modify price or original price based the conditions
	 */
	public function mwb_mrbpfw_role_based_price( $original_price, $product, $type ) {
		if ( ! is_user_logged_in() ) {
			$current_role = array( 'guest' );
		} else {
			$get_current_user_role = wp_get_current_user();
			$current_role          = $get_current_user_role->roles;
		}
		if ( class_exists( 'WC_Subscriptions_Product' ) && $product->is_subscription() ) {
			return $original_price;
		}
		$rule_apply  = get_option( 'mwb_mrbpfw_for_price_rule' );
		$get_options = get_option( 'user_setting_' . $current_role[0] );
		if ( 'simple' === $product->get_type() ) {
			if ( 'r_price' === $rule_apply ) {
				$new_price = $product->get_regular_price();
			} elseif ( 's_price' === $rule_apply ) {
				$new_price = $product->get_sale_price();
			}
		} elseif ( 'variable' === $product->get_type() ) {
			if ( 'r_price' === $rule_apply ) {
				$new_price = $product->get_regular_price();
			} elseif ( 's_price' === $rule_apply ) {
				$new_price = $product->get_sale_price();
			}
		} elseif ( 'variation' === $product->get_type() ) {
			if ( 'r_price' === $rule_apply ) {
				$new_price = $product->get_regular_price();
			} elseif ( 's_price' === $rule_apply ) {
				$new_price = $product->get_sale_price();
			}
		} else {
			return $original_price;
		}
		if ( empty( $new_price ) ) {
			return $original_price;
		}
		// Logic for calculating the new price here.
		$total_discount = $this->mwb_mrbpfw_get_discount_price( $product, $new_price );
		if ( empty( $new_price ) && $new_price < 0 ) {
			return wc_price( 0 );
		}
		$args1 = wp_parse_args(
			array(
				'qty'   => '',
				'price' => $product->get_sale_price(),
			)
		);
		$args2 = wp_parse_args(
			array(
				'qty'   => '',
				'price' => $product->get_regular_price(),
			)
		);
		if ( isset( $rule_apply ) && 's_price' === $rule_apply && $product->is_on_sale() ) {
			$sale_price_excl_tax    = wc_get_price_excluding_tax( $product, $args1 );
			$sale_price_incl_tax    = wc_get_price_including_tax( $product, $args1 );
			$regular_price_excl_tax = wc_get_price_excluding_tax( $product, $args2 );
			$regular_price_incl_tax = wc_get_price_including_tax( $product, $args2 );
		} elseif ( isset( $rule_apply ) && 'r_price' === $rule_apply && ( $product->is_on_sale() || ! $product->is_on_sale() ) ) {
			if ( $product->is_on_sale() ) {
				$sale_price_excl_tax = wc_get_price_excluding_tax( $product, $args1 );
				$sale_price_incl_tax = wc_get_price_including_tax( $product, $args1 );
			}
			$regular_price_excl_tax = wc_get_price_excluding_tax( $product, $args2 );
			$regular_price_incl_tax = wc_get_price_including_tax( $product, $args2 );
		}
		if ( isset( $get_options ) && ! empty( $get_options ) && in_array( 'show_tax_' . $current_role[0], $get_options, true ) ) {
			if ( isset( $rule_apply ) && 's_price' === $rule_apply ) {
				if ( $product->is_on_sale() ) {
					if ( isset( $total_discount ) && ! empty( $total_discount ) ) {
						$role_based_pricing = $sale_price_incl_tax - $total_discount;
					}
					$sale_price    = $sale_price_incl_tax;
					$regular_price = $regular_price_incl_tax;
				}
			} elseif ( isset( $rule_apply ) && 'r_price' === $rule_apply ) {
				if ( $product->is_on_sale() || ! $product->is_on_sale() ) {
					if ( isset( $total_discount ) && ! empty( $total_discount ) ) {
						$role_based_pricing = $regular_price_incl_tax - $total_discount;
					}
					if ( $product->is_on_sale() ) {
						$sale_price = $sale_price_incl_tax;
					}
					$regular_price = $regular_price_incl_tax;
				}
			}
		} else {
			if ( isset( $rule_apply ) && 's_price' === $rule_apply ) {
				if ( $product->is_on_sale() ) {
					if ( isset( $total_discount ) && ! empty( $total_discount ) ) {
						$role_based_pricing = $sale_price_excl_tax - $total_discount;
					}
					$sale_price    = $sale_price_excl_tax;
					$regular_price = $regular_price_excl_tax;
				}
			} elseif ( isset( $rule_apply ) && 'r_price' === $rule_apply ) {
				if ( $product->is_on_sale() || ! $product->is_on_sale() ) {
					if ( isset( $total_discount ) && ! empty( $total_discount ) ) {
						$role_based_pricing = $regular_price_excl_tax - $total_discount;
					}
					if ( $product->is_on_sale() ) {
						$sale_price = $sale_price_excl_tax;
					}
					$regular_price = $regular_price_excl_tax;
				}
			}
		}
		$tax_enable = get_option( 'woocommerce_calc_taxes' );
		$tax_label  = '';
		if ( 'yes' === $tax_enable ) {
			if ( isset( $get_options ) && ! empty( $get_options ) && in_array( 'show_tax_' . $current_role[0], $get_options, true ) ) {
				$tax_label = apply_filters( 'mwb_mrbpfw_tax_lable', __( 'incl.VAT', 'mwb-role-based-pricing-for-woocommerce' ) );
			} else {
				$tax_label = apply_filters( 'mwb_mrbpfw_tax_lable', __( 'excl.VAT', 'mwb-role-based-pricing-for-woocommerce' ) );
			}
		}
		if ( isset( $role_based_pricing ) && $role_based_pricing < 0 ) {
			return wc_price( 0 );
		}
		if ( function_exists( 'mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency' ) && ! is_admin() ) {
			if ( isset( $role_based_pricing ) ) {
				$role_based_pricing = mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency( '', $role_based_pricing );
			}
			if ( isset( $sale_price ) ) {
				$sale_price = mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency( '', $sale_price );
			}
			if ( isset( $regular_price ) ) {
				$regular_price = mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency( '', $regular_price );
			}
		}
		if ( isset( $total_discount ) && ! empty( $total_discount ) && ! empty( $get_options ) && in_array( 'role_based_price_' . $current_role[0], $get_options, true ) ) {
			if ( isset( $sale_price ) && $role_based_pricing > $sale_price && ! empty( $type ) && 'simple' === $type ) {
				return wc_price( $sale_price ) . ' ' . $tax_label;
			} else {
				return wc_price( $role_based_pricing ) . ' ' . $tax_label;
			}
		} elseif ( $product->is_on_sale() && ! empty( $get_options ) && in_array( 'on_sale_price_' . $current_role[0], $get_options, true ) ) {
			return wc_price( $sale_price ) . ' ' . $tax_label;
		} elseif ( $product->is_on_sale() && ! empty( $get_options ) && in_array( 'regular_price_' . $current_role[0], $get_options, true ) ) {
			return wc_price( $regular_price ) . ' ' . $tax_label;
		} elseif ( ! $product->is_on_sale() && ! empty( $get_options ) && in_array( 'regular_price_' . $current_role[0], $get_options, true ) ) {
			return wc_price( $regular_price ) . ' ' . $tax_label;
		} else {
			if ( isset( $get_options ) && ! empty( $get_options ) && in_array( 'show_tax_' . $current_role[0], $get_options, true ) ) {
				return wc_price( wc_get_price_including_tax( $product ) );
			} else {
				return wc_price( wc_get_price_excluding_tax( $product ) );
			}
		}
	}

	/**
	 * Function to update the cart item price based on the price rules.
	 *
	 * @param object $cart is the object of total cart item.
	 */
	public function mwb_mrbpfw_alter_price_cart( $cart ) {
		if ( is_admin() && ! defined( 'DOING_AJAX' ) ) {
			return;
		}
		foreach ( $cart->get_cart() as $cart_item_key => $cart_item ) {
			if ( ! is_user_logged_in() ) {
				$current_role = array( 'guest' );
			} else {
				$get_current_user_role = wp_get_current_user();
				$current_role          = $get_current_user_role->roles;
			}
			$product               = $cart_item['data'];
			$discount              = $this->mwb_mrbpfw_discount_availability_for_product( $product );
			$get_options           = get_option( 'user_setting_' . $current_role[0] );
			$rule_apply            = get_option( 'mwb_mrbpfw_for_price_rule' );
			if ( isset( $rule_apply ) && 's_price' === $rule_apply ) {
				if ( $product->is_on_sale() ) {
					$args1               = wp_parse_args(
						array(
							'qty'   => '',
							'price' => $product->get_sale_price(),
						)
					);
					$sale_price_excl_tax = wc_get_price_excluding_tax( $product, $args1 );
					$sale_price_incl_tax = wc_get_price_including_tax( $product, $args1 );
				}

				$args2                  = wp_parse_args(
					array(
						'qty'   => '',
						'price' => $product->get_regular_price(),
					)
				);
				$regular_price_excl_tax = wc_get_price_excluding_tax( $product, $args2 );
				$regular_price_incl_tax = wc_get_price_including_tax( $product, $args2 );
			} elseif ( isset( $rule_apply ) && 'r_price' === $rule_apply && ( $product->is_on_sale() || ! $product->is_on_sale() ) ) {
				if ( $product->is_on_sale() ) {
					$args1               = wp_parse_args(
						array(
							'qty'   => '',
							'price' => $product->get_sale_price(),
						)
					);
					$sale_price_excl_tax = wc_get_price_excluding_tax( $product, $args1 );
					$sale_price_incl_tax = wc_get_price_including_tax( $product, $args1 );
				}
				$args2                  = wp_parse_args(
					array(
						'qty'   => '',
						'price' => $product->get_regular_price(),
					)
				);
				$regular_price_excl_tax = wc_get_price_excluding_tax( $product, $args2 );
				$regular_price_incl_tax = wc_get_price_including_tax( $product, $args2 );
			}
			if ( 'simple' === $product->get_type() ) {
				if ( 'r_price' === $rule_apply ) {
					$new_price = $product->get_regular_price();
				} elseif ( 's_price' === $rule_apply ) {
					$new_price = $product->get_sale_price();
				}
			} elseif ( 'variable' === $product->get_type() ) {
				if ( 'r_price' === $rule_apply ) {
					$new_price = $product->get_regular_price();
				} elseif ( 's_price' === $rule_apply ) {
					$new_price = $product->get_sale_price();
				}
			} elseif ( 'variation' === $product->get_type() ) {
				if ( 'r_price' === $rule_apply ) {
					$new_price = $product->get_regular_price();
				} elseif ( 's_price' === $rule_apply ) {
					$new_price = $product->get_sale_price();
				}
			}
			if ( empty( $new_price ) ) {
				$new_price = $product->get_price();
			}
			// Logic for calculating the new price here.
			$total_discount = $this->mwb_mrbpfw_get_discount_price( $product, $new_price );
			if ( empty( $total_discount ) ) {
				$total_discount = 0;
			}
			if ( isset( $get_options ) && ! empty( $get_options ) && in_array( 'show_tax_' . $current_role[0], $get_options, true ) ) {
				if ( isset( $rule_apply ) && 's_price' === $rule_apply ) {
					if ( $product->is_on_sale() ) {
						if ( isset( $total_discount ) && ! empty( $total_discount ) ) {
							$role_based_pricing = $sale_price_incl_tax - $total_discount;
							$sale_price         = $sale_price_incl_tax;
						}
					}
					$regular_price = $regular_price_incl_tax;
				} elseif ( isset( $rule_apply ) && 'r_price' === $rule_apply ) {
					if ( $product->is_on_sale() || ! $product->is_on_sale() ) {
						if ( isset( $total_discount ) && ! empty( $total_discount ) ) {
							$role_based_pricing = $regular_price_incl_tax - $total_discount;
						}
						if ( $product->is_on_sale() ) {
							$sale_price = $sale_price_incl_tax;
						}
						$regular_price = $regular_price_incl_tax;
					}
				}
			} else {
				if ( isset( $rule_apply ) && 's_price' === $rule_apply ) {
					if ( $product->is_on_sale() ) {
						if ( isset( $total_discount ) && ! empty( $total_discount ) ) {
							$role_based_pricing = $sale_price_excl_tax - $total_discount;
							$sale_price         = $sale_price_excl_tax;
						}
					}
					$regular_price = $regular_price_excl_tax;
				} elseif ( isset( $rule_apply ) && 'r_price' === $rule_apply ) {
					if ( $product->is_on_sale() || ! $product->is_on_sale() ) {
						if ( isset( $total_discount ) && ! empty( $total_discount ) ) {
							$role_based_pricing = $regular_price_excl_tax - $total_discount;
						}
						if ( $product->is_on_sale() ) {
							$sale_price = $sale_price_excl_tax;
						}
						$regular_price = $regular_price_excl_tax;
					}
				}
			}
			if ( ! empty( $get_options ) && in_array( 'role_based_price_' . $current_role[0], $get_options, true ) && isset( $discount ) && 'yes' === $discount ) {
				if ( 'r_price' === $rule_apply ) {
					if ( $role_based_pricing < 0 ) {
						$cart_item['data']->set_price( 0 );
					} else {
						if ( ! empty( $sale_price ) && isset( $role_based_pricing ) && isset( $sale_price ) && $role_based_pricing > $sale_price ) {
							$cart_item['data']->set_price( $sale_price );
						} else {
							$cart_item['data']->set_price( $role_based_pricing );
						}
					}
				} elseif ( 's_price' === $rule_apply ) {
					if ( ! $product->is_on_sale() ) {
						$cart_item['data']->set_price( $regular_price );
					}
					if ( $product->is_on_sale() ) {
						if ( $role_based_pricing < 0 ) {
							$cart_item['data']->set_price( 0 );
						} else {
							$cart_item['data']->set_price( $role_based_pricing );
						}
					}
				}
			} elseif ( $product->is_on_sale() && ! empty( $get_options ) && in_array( 'on_sale_price_' . $current_role[0], $get_options, true ) ) {
				$cart_item['data']->set_price( $sale_price );
			} elseif ( $product->is_on_sale() && ! empty( $get_options ) && in_array( 'regular_price_' . $current_role[0], $get_options, true ) ) {
				$cart_item['data']->set_price( $regular_price );
			} elseif ( ! $product->is_on_sale() && ! empty( $get_options ) && in_array( 'regular_price_' . $current_role[0], $get_options, true ) ) {
				$cart_item['data']->set_price( $regular_price );
			} else {
				if ( isset( $get_options ) && ! empty( $get_options ) && in_array( 'show_tax_' . $current_role[0], $get_options, true ) ) {
					$cart_item['data']->set_price( wc_get_price_including_tax( $product ) );
				} else {
					$cart_item['data']->set_price( wc_get_price_excluding_tax( $product ) );
				}
			}
		}
		do_action( 'mwb_remove_tax_after_cart_table' );
	}
	/**
	 * Remove the tax from the cart page and checkout page.
	 */
	public function mwb_mrbpfw_remove_taxes() {
		if ( is_cart() || is_checkout() ) {
			add_filter( 'wc_tax_enabled', '__return_false' );
		}
	}

	/**
	 * Function to show the in-range for the varibale products.
	 *
	 * @param string $price is the current product price.
	 * @param object $product is the object of the current product.
	 * @return $price or $format_price price based on the conditions.
	 */
	public function mwb_mrbpfw_display_striked_out_price_for_variable( $price, $product ) {
		if ( is_admin() ) {
			return $price;
		}
		if ( ! is_user_logged_in() ) {
			$current_role = array( 'guest' );
		} else {
			$get_current_user_role = wp_get_current_user();
			$current_role          = $get_current_user_role->roles;
		}
		if ( class_exists( 'WC_Subscriptions_Product' ) && $product->is_subscription() ) {
			return $price;
		}
		$rule_apply  = get_option( 'mwb_mrbpfw_for_price_rule' );
		$get_options = get_option( 'user_setting_' . $current_role[0] );
		if ( isset( $get_options ) && empty( $get_options ) ) {
			return false;
		}
		$mrbpfw_regular_price_text    = get_option( 'mrbpfw_regular_price_text' );
		$mrbpfw_sale_price_text       = get_option( 'mrbpfw_sale_price_text' );
		$mrbpfw_role_based_price_text = get_option( 'mrbpfw_role_based_price_text' );
		$show_regular_price           = in_array( 'regular_price_' . $current_role[0], $get_options, true );
		$show_sale_price              = in_array( 'on_sale_price_' . $current_role[0], $get_options, true );
		if ( ( 's_price' === $rule_apply ) || ( 'r_price' === $rule_apply ) ) {
			$reg_price = '';
			if ( $product->is_type( 'variable' ) ) {
				$variations          = $product->get_children();
				$reg_prices          = array();
				$sale_prices         = array();
				$real_sale_prices    = array();
				$real_regular_prices = array();
				foreach ( $variations as $value ) {
					$single_variation = new WC_Product_Variation( $value );
					if ( isset( $rule_apply ) && 's_price' === $rule_apply ) {
						if ( $single_variation->is_on_sale() ) {
							$args1               = wp_parse_args(
								array(
									'qty'   => '',
									'price' => $single_variation->get_sale_price(),
								)
							);
							$sale_price_excl_tax = wc_get_price_excluding_tax( $single_variation, $args1 );
							$sale_price_incl_tax = wc_get_price_including_tax( $single_variation, $args1 );
						}
						$args2                  = wp_parse_args(
							array(
								'qty'   => '',
								'price' => $single_variation->get_regular_price(),
							)
						);
						$regular_price_excl_tax = wc_get_price_excluding_tax( $single_variation, $args2 );
						$regular_price_incl_tax = wc_get_price_including_tax( $single_variation, $args2 );
					} elseif ( isset( $rule_apply ) && 'r_price' === $rule_apply && ( $single_variation->is_on_sale() || ! $single_variation->is_on_sale() ) ) {
						if ( $product->is_on_sale() ) {
							$args1               = wp_parse_args(
								array(
									'qty'   => '',
									'price' => $single_variation->get_sale_price(),
								)
							);
							$sale_price_excl_tax = wc_get_price_excluding_tax( $single_variation, $args1 );
							$sale_price_incl_tax = wc_get_price_including_tax( $single_variation, $args1 );
						}
						$args2                  = wp_parse_args(
							array(
								'qty'   => '',
								'price' => $single_variation->get_regular_price(),
							)
						);
						$regular_price_excl_tax = wc_get_price_excluding_tax( $single_variation, $args2 );
						$regular_price_incl_tax = wc_get_price_including_tax( $single_variation, $args2 );
					}
					if ( ! $single_variation->is_on_sale() ) {
						$new_price = $this->mwb_mrbpfw_role_based_price( $price, $single_variation, '' );
						$new_price = explode( '</span>', $new_price );
						if ( isset( $new_price[1] ) ) {
							$new_price = floatval( preg_replace( '/[^-0-9\.]/', '', $new_price[1] ) );
							array_push( $reg_prices, $new_price );
						}
					}
					if ( $single_variation->is_on_sale() ) {
						$new_price = $this->mwb_mrbpfw_role_based_price( $price, $single_variation, '' );
						$new_price = explode( '</span>', $new_price );
						if ( isset( $new_price[1] ) ) {
							$new_price = floatval( preg_replace( '/[^-0-9\.]/', '', $new_price[1] ) );
							if ( 's_price' === $rule_apply ) {
								array_push( $sale_prices, $new_price );
							} elseif ( 'r_price' === $rule_apply ) {
								array_push( $reg_prices, $new_price );
							}
						}
					}
					if ( $product->is_on_sale() ) {
						if ( isset( $get_options ) && ! empty( $get_options ) && in_array( 'show_tax_' . $current_role[0], $get_options, true ) ) {
							if ( $single_variation->get_sale_price() ) {
								array_push( $real_sale_prices, $sale_price_incl_tax );
							}
							array_push( $real_regular_prices, $regular_price_incl_tax );
						} else {
							if ( $single_variation->get_sale_price() ) {
								array_push( $real_sale_prices, $sale_price_excl_tax );
							}
							array_push( $real_regular_prices, $regular_price_excl_tax );
						}
					}
					if ( ! $product->is_on_sale() ) {
						if ( isset( $get_options ) && ! empty( $get_options ) && in_array( 'show_tax_' . $current_role[0], $get_options, true ) ) {
							array_push( $real_regular_prices, $regular_price_incl_tax );
						} else {
							array_push( $real_regular_prices, $regular_price_excl_tax );
						}
					}
				}
				sort( $reg_prices );
				sort( $sale_prices );
				sort( $real_sale_prices );
				sort( $real_regular_prices );
				$tax_label = '';
				$tax_enable = get_option( 'woocommerce_calc_taxes' );
				if ( 'yes' === $tax_enable ) {
					if ( isset( $get_options ) && ! empty( $get_options ) && in_array( 'show_tax_' . $current_role[0], $get_options, true ) ) {
						$tax_label = apply_filters( 'mwb_mrbpfw_tax_lable', __( 'incl.VAT', 'mwb-role-based-pricing-for-woocommerce' ) );
					} else {
						$tax_label = apply_filters( 'mwb_mrbpfw_tax_lable', __( 'excl.VAT', 'mwb-role-based-pricing-for-woocommerce' ) );
					}
				}
				$discount = $this->mwb_mrbpfw_discount_availability_for_product( $product );
				if ( $product->is_on_sale() ) {
					if ( 's_price' === $rule_apply ) {
						$min_price = $sale_prices[0];
						$max_price = $sale_prices[ count( $sale_prices ) - 1 ];
					} elseif ( 'r_price' === $rule_apply ) {
						$min_price = $reg_prices[0];
						$max_price = $reg_prices[ count( $reg_prices ) - 1 ];
					}
					$min_sale_price = $real_sale_prices[0];
					$max_sale_price = $real_sale_prices[ count( $real_sale_prices ) - 1 ];
					$min_reg_price  = $real_regular_prices[0];
					$max_reg_price  = $real_regular_prices[ count( $real_regular_prices ) - 1 ];
					if ( function_exists( 'mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency' ) && ! is_admin() ) {
						if ( isset( $min_sale_price ) ) {
							$min_sale_price = mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency( '', $min_sale_price );
						}
						if ( isset( $max_sale_price ) ) {
							$max_sale_price = mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency( '', $max_sale_price );
						}
						if ( isset( $min_reg_price ) ) {
							$min_reg_price = mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency( '', $min_reg_price );
						}
						if ( isset( $max_reg_price ) ) {
							$max_reg_price = mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency( '', $max_reg_price );
						}
					}
					$min_sale_price1 = $min_sale_price;
					$min_price1      = $min_price;
					$min_price       = wc_price( $min_price );
					$max_price       = wc_price( $max_price );
					$min_sale_price  = wc_price( $min_sale_price );
					$max_sale_price  = wc_price( $max_sale_price );
					$min_reg_price   = wc_price( $min_reg_price );
					$max_reg_price   = wc_price( $max_reg_price );
					if ( $min_price === $max_price ) {
						if ( ! empty( $get_options ) && in_array( 'role_based_price_' . $current_role[0], $get_options, true ) && isset( $discount ) && 'yes' === $discount ) {
							$format_price = '';
							if ( ! empty( $show_regular_price ) && $show_regular_price ) {
								$reg_price    = $mrbpfw_regular_price_text . ' ' . $min_reg_price;
								$format_price = $this->mwb_mrbpfw_get_formatted_price_html( $reg_price . ' ' . $tax_label, true );
							}
							if ( ! empty( $show_sale_price ) && $show_sale_price ) {
								$sale_price    = $mrbpfw_sale_price_text . ' ' . $min_sale_price;
								if ( 'r_price' === $rule_apply && ! empty( $min_sale_price1 ) && $min_price1 > $min_sale_price1 ) {
									$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $min_sale_price1 . ' ' . $tax_label, false );
								} else {
									$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $min_sale_price1 . ' ' . $tax_label, true );
								}
							}
							$role_price    = $mrbpfw_role_based_price_text . ' ' . $min_price;
							if ( 'r_price' === $rule_apply && ! empty( $min_sale_price1 ) && $min_price1 > $min_sale_price1 ) {
								$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $role_price . ' ' . $tax_label, true );
							} else {
								$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $role_price . ' ' . $tax_label, false );
							}
							$format_price = apply_filters( 'mwb_rbpfw_addon_symbol', $format_price, $product );
							return $format_price;
						} elseif ( ! empty( $get_options ) && in_array( 'on_sale_price_' . $current_role[0], $get_options, true ) ) {
							$format_price = '';
							if ( ! empty( $show_regular_price ) || $show_regular_price ) {
								$reg_price    = wc_format_price_range( $mrbpfw_regular_price_text . ' ' . $min_reg_price, $mrbpfw_regular_price_text . ' ' . $max_reg_price );
								$format_price = $this->mwb_mrbpfw_get_formatted_price_html( $reg_price . ' ' . $tax_label, true );
							}
							$sale_price    = wc_format_price_range( $mrbpfw_sale_price_text . ' ' . $min_sale_price, $mrbpfw_sale_price_text . ' ' . $max_sale_price );
							$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $sale_price . ' ' . $tax_label, false );
							$format_price  = apply_filters( 'mwb_rbpfw_addon_symbol', $format_price, $product );
							return $format_price;

						} elseif ( ! empty( $get_options ) && in_array( 'regular_price_' . $current_role[0], $get_options, true ) ) {
							$reg_price    = wc_format_price_range( $mrbpfw_regular_price_text . ' ' . $min_reg_price, $mrbpfw_regular_price_text . ' ' . $max_reg_price );
							$format_price = $this->mwb_mrbpfw_get_formatted_price_html( $reg_price . ' ' . $tax_label, false );
							$format_price = apply_filters( 'mwb_rbpfw_addon_symbol', $format_price, $product );
							return $format_price;
						} else {
							return false;
						}
					} else {
						if ( ! empty( $get_options ) && in_array( 'role_based_price_' . $current_role[0], $get_options, true ) && isset( $discount ) && 'yes' === $discount ) {
							$format_price = '';
							if ( ! empty( $show_regular_price ) || $show_regular_price ) {
								$reg_price    = wc_format_price_range( $mrbpfw_regular_price_text . ' ' . $min_reg_price, $max_reg_price );
								$format_price = $this->mwb_mrbpfw_get_formatted_price_html( $reg_price . ' ' . $tax_label, true );
							}
							if ( ! empty( $show_sale_price ) || $show_sale_price ) {
								$sale_price    = wc_format_price_range( $mrbpfw_sale_price_text . ' ' . $min_sale_price, $max_sale_price );
								if ( 'r_price' === $rule_apply && ! empty( $min_sale_price1 ) && $min_price1 > $min_sale_price1 ) {
									$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $sale_price . ' ' . $tax_label, false );
								} else {
									$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $sale_price . ' ' . $tax_label, true );
								}
							}
							$role_price    = wc_format_price_range( $mrbpfw_role_based_price_text . ' ' . $min_price, $max_price );
							if ( 'r_price' === $rule_apply && ! empty( $min_sale_price1 ) && $min_price1 > $min_sale_price1 ) {
								$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $role_price . ' ' . $tax_label, true );
							} else {
								$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $role_price . ' ' . $tax_label, false );
							}
							$format_price = apply_filters( 'mwb_rbpfw_addon_symbol', $format_price, $product );
							return $format_price;
						} elseif ( ! empty( $get_options ) && in_array( 'on_sale_price_' . $current_role[0], $get_options, true ) ) {
							$format_price = '';
							if ( ! empty( $show_regular_price ) || $show_regular_price ) {
								$reg_price    = wc_format_price_range( $mrbpfw_regular_price_text . ' ' . $min_reg_price, $max_reg_price );
								$format_price = $this->mwb_mrbpfw_get_formatted_price_html( $reg_price . ' ' . $tax_label, true );
							}
							$sale_price    = wc_format_price_range( $mrbpfw_sale_price_text . ' ' . $min_sale_price, $max_sale_price );
							$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $sale_price . ' ' . $tax_label, false );
							$format_price  = apply_filters( 'mwb_rbpfw_addon_symbol', $format_price, $product );
							return $format_price;
						} elseif ( ! empty( $get_options ) && in_array( 'regular_price_' . $current_role[0], $get_options, true ) ) {
							$reg_price    = wc_format_price_range( $mrbpfw_regular_price_text . ' ' . $min_reg_price, $max_reg_price );
							$format_price = $this->mwb_mrbpfw_get_formatted_price_html( $reg_price . ' ' . $tax_label, false );
							$format_price = apply_filters( 'mwb_rbpfw_addon_symbol', $format_price, $product );
							return $format_price;
						} else {
							return false;
						}
					}
				} elseif ( ! $product->is_on_sale() ) {
					$min_price     = $reg_prices[0];
					$max_price     = $reg_prices[ count( $reg_prices ) - 1 ];
					$min_reg_price = $real_regular_prices[0];
					$max_reg_price = $real_regular_prices[ count( $real_regular_prices ) - 1 ];
					if ( function_exists( 'mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency' ) && ! is_admin() ) {
						if ( isset( $min_reg_price ) ) {
							$min_reg_price = mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency( '', $min_reg_price );
						}
						if ( isset( $max_reg_price ) ) {
							$max_reg_price = mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency( '', $max_reg_price );
						}
					}
					$min_price     = wc_price( $min_price );
					$max_price     = wc_price( $max_price );
					$min_reg_price = wc_price( $min_reg_price );
					$max_reg_price = wc_price( $max_reg_price );
					if ( 's_price' === $rule_apply ) {
						if ( $min_reg_price === $max_reg_price ) {
							$format_price  = '';
							$role_price    = $mrbpfw_role_based_price_text . ' ' . $min_reg_price;
							$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $role_price . ' ' . $tax_label, false );
							return $format_price;
						} else {
							if ( ! empty( $show_regular_price ) && $show_regular_price ) {
								$format_price  = '';
								$role_price    = $mrbpfw_regular_price_text . ' ' . $min_reg_price . ' - ' . $max_reg_price;
								$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $role_price . ' ' . $tax_label, false );
								return $format_price;
							}
						}
					}
					if ( $min_price === $max_price ) {
						if ( ! empty( $get_options ) && in_array( 'role_based_price_' . $current_role[0], $get_options, true ) && isset( $discount ) && 'yes' === $discount ) {
							$format_price = '';
							if ( ! empty( $show_regular_price ) || $show_regular_price ) {
								$reg_price    = $mrbpfw_regular_price_text . ' ' . $min_reg_price;
								$format_price = $this->mwb_mrbpfw_get_formatted_price_html( $reg_price . ' ' . $tax_label, true );
							}
							$role_price    = $mrbpfw_role_based_price_text . ' ' . $min_price;
							$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $role_price . ' ' . $tax_label, false );
							$format_price  = apply_filters( 'mwb_rbpfw_addon_symbol', $format_price, $product );
							return $format_price;
						} elseif ( ! empty( $get_options ) && in_array( 'regular_price_' . $current_role[0], $get_options, true ) ) {
							$reg_price    = $mrbpfw_regular_price_text . ' ' . $min_reg_price;
							$format_price = $this->mwb_mrbpfw_get_formatted_price_html( $reg_price . ' ' . $tax_label, false );
							$format_price = apply_filters( 'mwb_rbpfw_addon_symbol', $format_price, $product );
							return $format_price;
						} else {
							return false;
						}
					} else {
						if ( ! empty( $get_options ) && in_array( 'role_based_price_' . $current_role[0], $get_options, true ) && isset( $discount ) && 'yes' === $discount ) {
							$format_price = '';
							if ( ! empty( $show_regular_price ) || $show_regular_price ) {
								$reg_price    = wc_format_price_range( $mrbpfw_regular_price_text . ' ' . $min_reg_price, $max_reg_price );
								$format_price = $this->mwb_mrbpfw_get_formatted_price_html( $reg_price . ' ' . $tax_label, true );
							}
							$role_price    = wc_format_price_range( $mrbpfw_role_based_price_text . ' ' . $min_price, $max_price );
							$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $role_price . ' ' . $tax_label, false );
							$format_price = apply_filters( 'mwb_rbpfw_addon_symbol', $format_price, $product );
							return $format_price;
						} elseif ( ! empty( $get_options ) && in_array( 'regular_price_' . $current_role[0], $get_options, true ) ) {
							$reg_price    = $mrbpfw_regular_price_text . ' ' . $min_reg_price;
							$format_price = $this->mwb_mrbpfw_get_formatted_price_html( $reg_price . ' ' . $tax_label, false );
							$format_price = apply_filters( 'mwb_rbpfw_addon_symbol', $format_price, $product );
							return $format_price;
						} else {
							return false;
						}
					}
				}
			}
			if ( $product->is_type( 'simple' ) ) {
				return $this->mwb_mrbpfw_get_formatted_price( $price, $product );
			} else {
				return $price;
			}
		} else {
			return $price;
		}
	}

	/**
	 * Function to show to discount fixed or percentage.
	 */
	public function mwb_mrbpfw_show_sale_percentage_loop() {
		global $product;
		if ( ! is_user_logged_in() ) {
			$current_role = array( 'guest' );
		} else {
			$get_current_user_role = wp_get_current_user();
			$current_role          = $get_current_user_role->roles;
		}
		$rule_apply                = get_option( 'mwb_mrbpfw_for_price_rule' );
		$check_show_total_discount = get_option( 'user_setting_' . $current_role[0] );
		$product_id                = $product->get_id();
		if ( ( 'r_price' === $rule_apply ) || ( 's_price' === $rule_apply && $product->is_on_sale() ) ) {
			$discount = $this->mwb_mrbpfw_discount_availability_for_product( $product );
		}
		$get_options = get_option( 'user_setting_' . $current_role[0] );
		if ( ! empty( $check_show_total_discount ) && in_array( 'role_based_price_' . $current_role[0], $get_options, true ) && in_array( 'show_total_discount_' . $current_role[0], $check_show_total_discount, true ) && isset( $discount ) && 'yes' === $discount ) {
			// Logic for calculating the new price here.
			$all_rules_ids         = $this->mwb_mrbpfw_get_all_rule_ids_for_current_role();
			$priority_wise_rule_id = array();
			if ( 'simple' === $product->get_type() || 'variable' === $product->get_type() ) {
				if ( isset( $all_rules_ids ) && ! empty( $all_rules_ids ) ) {
					foreach ( $all_rules_ids as $key => $id ) {
						$priority                           = get_post_meta( $id, 'mwb_mrbpfw_priority', true );
						$priority_wise_rule_id[ $priority ] = $id;
					}
					ksort( $priority_wise_rule_id );
					$count = 0;
					foreach ( $priority_wise_rule_id as $key => $id ) {
						++$count;
						$rule_type       = get_post_meta( $id, 'mwb_mrbpfw_rule_type', true );
						$discount_type   = get_post_meta( $id, 'mwb_mrbpfw_discount_type', true );
						$price           = get_post_meta( $id, 'mwb_mrbpfw_price', true );
						$all_product_ids = get_post_meta( $id, 'mwb_mrbpfw_all_products', true );
						if ( isset( $rule_type ) && ! empty( $rule_type ) && 'all_products' === $rule_type && ( empty( $all_product_ids ) || ( ! empty( $all_product_ids ) && in_array( $product->get_id(), $all_product_ids, false ) ) ) ) {
							if ( isset( $price ) && ! empty( $price ) ) {
								if ( 'fixed' === $discount_type ) {
									if ( function_exists( 'mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency' ) && ! is_admin() ) {
										if ( isset( $price ) ) {
											$price = mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency( '', $price );
										}
									}
									echo wp_kses_post( apply_filters( 'mwb_total_fixed_discount_html', '<div class="sale-perc">' . __( 'Total Discount ', 'mwb-role-based-pricing-for-woocommerce' ) . wc_price( $price ) . '</div>' ) );
									break;
								} elseif ( 'percentage' === $discount_type ) {
									echo wp_kses_post( apply_filters( 'mwb_total_percentage_discount_html', '<div class="sale-perc">' . esc_html( $price ) . __( '% Discount', 'mwb-role-based-pricing-for-woocommerce' ) . '</div>' ) );
									break;
								}
							}
						} elseif ( isset( $rule_type ) && ! empty( $rule_type ) && 'categories' === $rule_type ) {
							$categories = get_post_meta( $id, 'mwb_mrbpfw_categories', true );
							if ( isset( $categories ) && ! empty( $categories ) ) {
								if ( has_term( $categories, 'product_cat', $product->get_id() ) ) {
									if ( isset( $price ) && ! empty( $price ) ) {
										if ( 'fixed' === $discount_type ) {
											if ( function_exists( 'mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency' ) && ! is_admin() ) {
												if ( isset( $price ) ) {
													$price = mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency( '', $price );
												}
											}
											echo wp_kses_post( apply_filters( 'mwb_total_fixed_discount_html', '<div class="sale-perc">' . __( 'Total Discount ', 'mwb-role-based-pricing-for-woocommerce' ) . wc_price( $price ) . '</div>' ) );
											break;
										} elseif ( 'percentage' === $discount_type ) {
											echo wp_kses_post( apply_filters( 'mwb_total_percentage_discount_html', '<div class="sale-perc">' . esc_html( $price ) . __( '% Discount', 'mwb-role-based-pricing-for-woocommerce' ) . '</div>' ) );
											break;
										}
									}
								}
							}
						} elseif ( isset( $rule_type ) && ! empty( $rule_type ) && 'tags' === $rule_type ) {
							$tags = get_post_meta( $id, 'mwb_mrbpfw_tags', true );
							if ( isset( $tags ) && ! empty( $tags ) ) {
								if ( has_term( $tags, 'product_tag', $product->get_id() ) ) {
									if ( isset( $price ) && ! empty( $price ) ) {
										if ( 'fixed' === $discount_type ) {
											if ( function_exists( 'mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency' ) && ! is_admin() ) {
												if ( isset( $price ) ) {
													$price = mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency( '', $price );
												}
											}
											echo wp_kses_post( apply_filters( 'mwb_total_fixed_discount_html', '<div class="sale-perc">' . __( 'Total Discount ', 'mwb-role-based-pricing-for-woocommerce' ) . wc_price( $price ) . '</div>' ) );
											break;
										} elseif ( 'percentage' === $discount_type ) {
											echo wp_kses_post( apply_filters( 'mwb_total_percentage_discount_html', '<div class="sale-perc">' . esc_html( $price ) . __( '% Discount', 'mwb-role-based-pricing-for-woocommerce' ) . '</div>' ) );
											break;
										}
									}
								}
							}
						}
						if ( 1 === $count ) {
							break;
						}
					}
				}
			}
		}
	}

	/**
	 * Function Hide the add to cart button from frontend.
	 */
	public function mwb_mrbpfw_add_to_cart_button_hide() {
		if ( ! is_user_logged_in() ) {
			$current_role = array( 'guest' );
		} else {
			$get_current_user_role = wp_get_current_user();
			$current_role          = $get_current_user_role->roles;
		}
		$get_option = get_option( 'user_setting_' . $current_role[0] );
		if ( ! empty( $current_role ) && ! empty( $get_option ) && ! in_array( 'add_to_cart_' . $current_role[0], $get_option, true ) ) {
			remove_action( 'woocommerce_after_shop_loop_item', 'woocommerce_template_loop_add_to_cart', 10 );
			remove_action( 'woocommerce_single_product_summary', 'woocommerce_template_single_add_to_cart', 30 );
		}
	}

	/**
	 * Function to add or not <del> tag for price.
	 *
	 * @param string  $price_html is the current price html.
	 * @param boolean $del false or true to addd or not the <del> tag.
	 */
	public function mwb_mrbpfw_get_formatted_price_html( $price_html, $del = false ) {
		$price_formatted_html = '';
		if ( ! ( empty( $price_html ) ) ) {
			$price_formatted_html  = '<span class="format-price">';
			$price_formatted_html .= $del ? '<del>' : '';
			$price_formatted_html .= $price_html;
			$price_formatted_html .= $del ? '</del>' : '';
			$price_formatted_html .= '</span>';
		}
		return apply_filters( 'mwb_price_format_html', $price_formatted_html );
	}

	/**
	 * Function to format the price based on the text setting.
	 *
	 * @param string $original_price is the current product price based on the rule applied.
	 * @param object $product is the current product object.
	 * @return string the price based on the rule applied.
	 */
	public function mwb_mrbpfw_get_formatted_price( $original_price, $product ) {
		if ( ! is_user_logged_in() ) {
			$current_role = array( 'guest' );
		} else {
			$get_current_user_role = wp_get_current_user();
			$current_role          = $get_current_user_role->roles;
		}
		$get_option = get_option( 'user_setting_' . $current_role[0] );
		if ( isset( $get_option ) && empty( $get_option ) ) {
			return false;
		}
		$rule_apply = get_option( 'mwb_mrbpfw_for_price_rule' );
		if ( isset( $rule_apply ) && 's_price' === $rule_apply ) {
			if ( $product->is_on_sale() ) {
				$args1               = wp_parse_args(
					array(
						'qty'   => '',
						'price' => $product->get_sale_price(),
					)
				);
				$sale_price_excl_tax = wc_get_price_excluding_tax( $product, $args1 );
				$sale_price_incl_tax = wc_get_price_including_tax( $product, $args1 );
			}
			$args2                  = wp_parse_args(
				array(
					'qty'   => '',
					'price' => $product->get_regular_price(),
				)
			);
			$regular_price_excl_tax = wc_get_price_excluding_tax( $product, $args2 );
			$regular_price_incl_tax = wc_get_price_including_tax( $product, $args2 );
		} elseif ( isset( $rule_apply ) && 'r_price' === $rule_apply && ( $product->is_on_sale() || ! $product->is_on_sale() ) ) {
			if ( $product->is_on_sale() ) {
				$args1               = wp_parse_args(
					array(
						'qty'   => '',
						'price' => $product->get_sale_price(),
					)
				);
				$sale_price_excl_tax = wc_get_price_excluding_tax( $product, $args1 );
				$sale_price_incl_tax = wc_get_price_including_tax( $product, $args1 );
			}
			$args2                  = wp_parse_args(
				array(
					'qty'   => '',
					'price' => $product->get_regular_price(),
				)
			);
			$regular_price_excl_tax = wc_get_price_excluding_tax( $product, $args2 );
			$regular_price_incl_tax = wc_get_price_including_tax( $product, $args2 );
		}
		if ( $product->is_type( 'simple' ) ) {
			$mrbpfw_regular_price_text    = get_option( 'mrbpfw_regular_price_text' );
			$mrbpfw_sale_price_text       = get_option( 'mrbpfw_sale_price_text' );
			$mrbpfw_role_based_price_text = get_option( 'mrbpfw_role_based_price_text' );
			$rule_apply                   = get_option( 'mwb_mrbpfw_for_price_rule' );
			$get_options                  = get_option( 'user_setting_' . $current_role[0] );
			$show_regular_price           = in_array( 'regular_price_' . $current_role[0], $get_options, true );
			$show_sale_price              = in_array( 'on_sale_price_' . $current_role[0], $get_options, true );
			if ( isset( $get_options ) && ! empty( $get_options ) && in_array( 'show_tax_' . $current_role[0], $get_options, true ) ) {
				if ( isset( $rule_apply ) && 's_price' === $rule_apply ) {
					if ( $product->is_on_sale() ) {
						$sale_price = $sale_price_incl_tax;
					}
					$regular_price = $regular_price_incl_tax;
				} elseif ( isset( $rule_apply ) && 'r_price' === $rule_apply ) {
					if ( $product->is_on_sale() || ! $product->is_on_sale() ) {
						if ( $product->is_on_sale() ) {
							$sale_price = $sale_price_incl_tax;
						}
						$regular_price = $regular_price_incl_tax;
					}
				}
			} else {
				if ( isset( $rule_apply ) && 's_price' === $rule_apply ) {
					if ( $product->is_on_sale() ) {
						$sale_price = $sale_price_excl_tax;
					}
					$regular_price = $regular_price_excl_tax;
				} elseif ( isset( $rule_apply ) && 'r_price' === $rule_apply ) {
					if ( $product->is_on_sale() || ! $product->is_on_sale() ) {
						if ( $product->is_on_sale() ) {
							$sale_price = $sale_price_excl_tax;
						}
						$regular_price = $regular_price_excl_tax;
					}
				}
			}
			$tax_label = '';
			$tax_enable = get_option( 'woocommerce_calc_taxes' );
			if ( 'yes' === $tax_enable ) {
				if ( isset( $get_options ) && ! empty( $get_options ) && in_array( 'show_tax_' . $current_role[0], $get_options, true ) ) {
					$tax_label = apply_filters( 'mwb_mrbpfw_tax_lable', __( 'incl.VAT', 'mwb-role-based-pricing-for-woocommerce' ) );
				} else {
					$tax_label = apply_filters( 'mwb_mrbpfw_tax_lable', __( 'excl.VAT', 'mwb-role-based-pricing-for-woocommerce' ) );
				}
			}
			if ( function_exists( 'mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency' ) && ! is_admin() ) {
				if ( isset( $regular_price ) ) {
					$regular_price = mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency( '', $regular_price );
				}
				if ( isset( $sale_price ) ) {
					$sale_price = mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency( '', $sale_price );
				}
				if ( isset( $price ) ) {
					$price = mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency( '', $price );
				}
			}
			$discount = $this->mwb_mrbpfw_discount_availability_for_product( $product );
			if ( ! empty( $get_options ) && in_array( 'role_based_price_' . $current_role[0], $get_options, true ) && isset( $discount ) && 'yes' === $discount ) {
				$price = $this->mwb_mrbpfw_role_based_price( $original_price, $product, '' );
				$new_price = explode( '</span>', $price );
				if ( isset( $new_price[1] ) ) {
					$price_role = floatval( preg_replace( '/[^-0-9\.]/', '', $new_price[1] ) );
				}
				if ( ! empty( $price ) && ( $product->is_type( 'simple' ) ) ) {
					if ( 's_price' === $rule_apply ) {
						if ( $product->is_on_sale() ) {
							$format_price = '';
							if ( ! empty( $show_regular_price ) || $show_regular_price ) {
								$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $mrbpfw_regular_price_text . ' ' . wc_price( $regular_price ) . ' ' . $tax_label, true );
							}
							if ( ! empty( $show_sale_price ) || $show_sale_price ) {
								$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $mrbpfw_sale_price_text . ' ' . wc_price( $sale_price ) . ' ' . $tax_label, true );
							}
							$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $mrbpfw_role_based_price_text . ' ' . $price, false );
							$format_price  = apply_filters( 'mwb_rbpfw_addon_symbol', $format_price, $product );
							return $format_price;
						} elseif ( ! $product->is_on_sale() ) {
							if ( ! empty( $show_regular_price ) && $show_regular_price ) {
								$format_price = $this->mwb_mrbpfw_get_formatted_price_html( $mrbpfw_regular_price_text . ' ' . wc_price( $regular_price ) . ' ' . $tax_label, false );
								$format_price = apply_filters( 'mwb_rbpfw_addon_symbol', $format_price, $product );
								return $format_price;
							} else {
								return false;
							}
						}
					} elseif ( 'r_price' === $rule_apply && ( $product->is_on_sale() || ! $product->is_on_sale() ) ) {
						if ( $product->is_on_sale() ) {
							$format_price = '';
							if ( ! empty( $show_regular_price ) && $show_regular_price ) {
								$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $mrbpfw_regular_price_text . ' ' . wc_price( $regular_price ) . ' ' . $tax_label, true );
							}
							if ( ! empty( $show_sale_price ) && isset( $price_role ) && $price_role > $sale_price ) {
								$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $mrbpfw_sale_price_text . ' ' . wc_price( $sale_price ) . ' ' . $tax_label, false );
							} else {
								$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $mrbpfw_sale_price_text . ' ' . wc_price( $sale_price ) . ' ' . $tax_label, true );
							}
							if ( isset( $sale_price ) && isset( $price_role ) && $price_role > $sale_price ) {
								$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $mrbpfw_role_based_price_text . ' ' . $price, true );
							} else {
								$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $mrbpfw_role_based_price_text . ' ' . $price, false );
							}
							$format_price = apply_filters( 'mwb_rbpfw_addon_symbol', $format_price, $product );
							return $format_price;
						} elseif ( ! $product->is_on_sale() ) {
							$format_price = '';
							if ( ! empty( $show_regular_price ) && $show_regular_price ) {
								$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $mrbpfw_regular_price_text . ' ' . wc_price( $regular_price ) . ' ' . $tax_label, true );
							}
							$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $mrbpfw_role_based_price_text . ' ' . $price, false );
							$format_price  = apply_filters( 'mwb_rbpfw_addon_symbol', $format_price, $product );
							return $format_price;
						}
					} else {
						return $price;
					}
				} else {
					return $original_price;
				}
			}
			if ( $product->is_on_sale() && ! empty( $get_option ) && in_array( 'on_sale_price_' . $current_role[0], $get_option, true ) ) {
				$format_price = '';
				if ( ! empty( $show_regular_price ) && $show_regular_price ) {
					$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $mrbpfw_regular_price_text . ' ' . wc_price( $regular_price ) . ' ' . $tax_label, true );
				}
				$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $mrbpfw_sale_price_text . ' ' . wc_price( $sale_price ) . ' ' . $tax_label, false );
				$format_price  = apply_filters( 'mwb_rbpfw_addon_symbol', $format_price, $product );
				return $format_price;
			}
			if ( $product->is_on_sale() && ! empty( $get_option ) && in_array( 'regular_price_' . $current_role[0], $get_option, true ) ) {
				$format_price = '';
				if ( ! empty( $show_regular_price ) || $show_regular_price ) {
					$format_price .= $this->mwb_mrbpfw_get_formatted_price_html( $mrbpfw_regular_price_text . ' ' . wc_price( $regular_price ) . ' ' . $tax_label, false );
					$format_price  = apply_filters( 'mwb_rbpfw_addon_symbol', $format_price, $product );
					return $format_price;
				} else {
					return false;
				}
			}
			if ( ! $product->is_on_sale() && ! empty( $get_option ) && in_array( 'regular_price_' . $current_role[0], $get_option, true ) ) {
				$format_price = $this->mwb_mrbpfw_get_formatted_price_html( $mrbpfw_regular_price_text . ' ' . wc_price( $regular_price ) . ' ' . $tax_label, false );
				$format_price = apply_filters( 'mwb_rbpfw_addon_symbol', $format_price, $product );
				return $format_price;
			} else {
				return false;
			}
		} else {
			return $original_price;
		}
	}

	/**
	 * Hide the Add to cart button if condition not matched.
	 *
	 * @param boolean $value is boolean.
	 * @param object  $product is the current product.
	 */
	public function mwb_mrbpfw_remove_add_to_cart_button( $value, $product ) {
		if ( ! is_user_logged_in() ) {
			$current_role = array( 'guest' );
		} else {
			$get_current_user_role = wp_get_current_user();
			$current_role          = $get_current_user_role->roles;
		}
		$get_options = get_option( 'user_setting_' . $current_role[0] );
		if ( 'OceanWP' === wp_get_theme()->name && ( empty( $get_options ) || ( ! empty( $get_options ) && ! in_array( 'add_to_cart_' . $current_role[0], $get_options, true ) ) ) ) {
			return false;
		} elseif ( 'Avada' === wp_get_theme()->name && ( empty( $get_options ) || ( ! empty( $get_options ) && ! in_array( 'add_to_cart_' . $current_role[0], $get_options, true ) ) ) ) {
			return false;
		} elseif ( 'Betheme' === wp_get_theme()->name && ( empty( $get_options ) || ( ! empty( $get_options ) && ! in_array( 'add_to_cart_' . $current_role[0], $get_options, true ) ) ) ) {
			return false;
		} elseif ( 'Flatsome' === wp_get_theme()->name && ( empty( $get_options ) || ( ! empty( $get_options ) && ! in_array( 'add_to_cart_' . $current_role[0], $get_options, true ) ) ) ) {
			return false;
		}
		$rule_apply = get_option( 'mwb_mrbpfw_for_price_rule' );
		if ( isset( $get_options ) && ! empty( $get_options ) && ! in_array( 'regular_price_' . $current_role[0], $get_options, true ) && ! in_array( 'on_sale_price_' . $current_role[0], $get_options, true ) && ! in_array( 'role_based_price_' . $current_role[0], $get_options, true ) ) {
			return false;
		} elseif ( ! empty( $current_role ) && ! empty( $get_options ) && 's_price' === $rule_apply && ! $product->is_on_sale() && ( ! in_array( 'regular_price_' . $current_role[0], $get_options, true ) ) ) {
			return false;
		} elseif ( ! empty( $current_role ) && ! empty( $get_options ) && 'r_price' === $rule_apply && ! $product->is_on_sale() && ( ! in_array( 'regular_price_' . $current_role[0], $get_options, true ) && in_array( 'on_sale_price_' . $current_role[0], $get_options, true ) && ! in_array( 'role_based_price_' . $current_role[0], $get_options, true ) ) ) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 * Subscription compatibility code on the product.
	 *
	 * @param string $price .
	 * @param object $product .
	 */
	public function mwb_rbpfw_addon_symbol( $price, $product ) {
		if ( function_exists( 'mwb_sfw_check_product_is_subscription' ) && mwb_sfw_check_product_is_subscription( $product ) && function_exists( 'mwb_sfw_get_time_interval_for_price' ) ) {
			$mwb_recurring_number   = get_post_meta( $product->get_id(), 'mwb_sfw_subscription_number', true );
			$mwb_recurring_interval = get_post_meta( $product->get_id(), 'mwb_sfw_subscription_interval', true );
			$mwb_price_html         = mwb_sfw_get_time_interval_for_price( $mwb_recurring_number, $mwb_recurring_interval );
			return $price . ' / ' . $mwb_price_html;
		} else {
			return $price;
		}
	}

	/**
	 * Subscription compatibility code on the cart.
	 *
	 * @param string $product_price .
	 * @param object $cart_item .
	 */
	public function mwb_rbpfw_cart_price( $product_price, $cart_item ) {
		$product      = $cart_item['data'];
		$price_change = $this->mwb_mrbpfw_role_based_price( $product_price, $product, 'simple' );
		$new_price = explode( '</span>', $price_change );
		if ( isset( $new_price[1] ) ) {
			if ( ! is_user_logged_in() ) {
				$current_role = array( 'guest' );
			} else {
				$get_current_user_role = wp_get_current_user();
				$current_role          = $get_current_user_role->roles;
			}
			$get_options = get_option( 'user_setting_' . $current_role[0] );
			$tax_enable  = get_option( 'woocommerce_calc_taxes' );
			if ( isset( $get_options ) && ! empty( $get_options ) && in_array( 'show_tax_' . $current_role[0], $get_options, true ) && 'yes' === $tax_enable ) {
				$incr = 1;
				if ( function_exists( 'mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency' ) && ! is_admin() ) {
					$incr = mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency( '', $incr );
				}
			} else {
				$incr = 0;
			}
			$price_change = floatval( preg_replace( '/[^-0-9\.]/', '', $new_price[1] ) ) + $incr;
			$price_change = wc_price( $price_change );
		}
		if ( isset( $price_change ) ) {
			return $price_change;
		} else {
			return $price;
		}
	}
	/**
	 * Tax Label
	 *
	 * @param string $tax_label .
	 */
	public function mwb_mrbpfw_tax_label( $tax_label ) {
		$tax_label_setting = get_option( 'mrbpfw_tax_label' );
		if ( ! empty( $tax_label_setting ) && 'on' == $tax_label_setting ) {
			return false;
		} else {
			return $tax_label;
		}
	}
}
